var $ = require('jquery');
var AppConstants = require('../constants/AppConstants');
var AppDispatcher = require('../dispatchers/AppDispatcher');
var EventEmitter = require('events').EventEmitter;
var HelixWebServicesClient = require('helix_web_services_client');
var React = require('react');
var assign = require('object-assign');

var LOGGED_IN = 'logged_in';
var LOGGED_OUT = 'logged_out';

// The session store handles our security token session.
var Session = assign({}, EventEmitter.prototype, {
  hasSession: function() {
    return this.getSession() != null;
  },

  getSession: function() {
    return JSON.parse(localStorage.getItem('session'));
  },

  getClient: function() {
    if (this.client == null && this.hasSession()) {
      var session = this.getSession();
      this.client = new HelixWebServicesClient({
        prefix: '/hws',
        user: session.user,
        session_token: session.token
      });
      var self = this;
      this.client.addSessionExpiredHandler(function() {
        localStorage.removeItem('session');
        self.client = null;
        self.emitLoggedOut();
      });
    }
    return this.client;
  },

  logIn: function(user, password) {
    var client = new HelixWebServicesClient({
      prefix: '/hws',
      user: user
    });
    var ajax = client.logIn(password);

    var self = this;

    ajax.done(function(token, status) {
      localStorage.setItem('session', JSON.stringify({ user: user, token: token }));
      self.emitLoggedIn();
    });

    ajax.fail(function() {
      console.log('fail!', arguments);
    });
  },

  logOut: function() {
    var client = this.getClient();
    if (client) {
      var ajax = client.logOut();
      var self = this;
      var finish = function() {
        localStorage.removeItem('session');
        self.client = null;
        self.emitLoggedOut();
      };

      ajax.done(finish);
      ajax.fail(finish);
    }
  },



  emitLoggedIn: function() {
    this.emit(LOGGED_IN);
  },

  emitLoggedOut: function() {
    this.emit(LOGGED_OUT);
  },

  addLoggedInListener: function(callback) {
    this.on(LOGGED_IN, callback);
  },

  removeLoggedInListener: function(callback) {
    this.removeListener(LOGGED_IN, callback);
  },

  addLoggedOutListener: function(cb) {
    this.on(LOGGED_OUT, cb);
  },

  removeLoggedOutListener: function(cb) {
    this.removeListener(LOGGED_OUT, cb);
  }
});

AppDispatcher.register(function(action) {
  switch(action.actionType) {
    case AppConstants.LOG_IN:
      Session.logIn(action.user, action.password);
      break;

    case AppConstants.LOG_OUT:
      Session.logOut();
      break;

    default:
      // Do nothing
  }
});

module.exports = Session;
